#!/usr/bin/env node

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.watch = watch;
exports.converter = converter;
exports.validator = validator;
exports.validateAction = validateAction;
exports.convertAction = convertAction;
exports.startBunsen = startBunsen;

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _logger = require('../lib/logger');

var _logger2 = _interopRequireDefault(_logger);

var _converterUis = require('../lib/converter-uis1');

var _converterBv = require('..//lib/converter-bv1');

var _validate = require('../lib/validate');

var _utils = require('../lib/utils');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsPromise = require('fs-promise');

var _fsPromise2 = _interopRequireDefault(_fsPromise);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _strings = require('../assets/strings');

require('colors');

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function watch(file, callback, watching) {
  if (watching) {
    console.log('watching');
    _fs2.default.watch(file, { encoding: 'buffer' }, callback);
  }
}

/**
 * convert a legacy view to bv1
 * @param {String} inFile the file to read in
 * @param {String} outFile an optional output path
 * @param {Object} logger a logger instance (for logging)
 * @returns {Object<Promise>} a promise
 */
function converter(inFile, outFile, logger) {
  outFile = outFile || (0, _utils.getDefaultOutputPath)(inFile);
  return _fsPromise2.default.readFile(inFile).then(function (legacyViewJSON) {
    return (0, _utils.parseJSON)(legacyViewJSON);
  }).then(function (legacyView) {
    return (0, _utils.getLegacyViewType)(legacyView).then(function (legacyViewType) {
      switch (legacyViewType) {
        case 'uis1':
          return (0, _converterUis.convert)(legacyView, outFile, logger);
          break;
        case 'bv1':
          console.log((0, _converterBv.convert)(legacyView, outFile, logger));
          return (0, _converterBv.convert)(legacyView, outFile, logger);
      }
    });
  }).then(function (uis2View) {
    return _fsPromise2.default.writeFile(outFile, JSON.stringify(uis2View, null, 2));
  }).then(function (result) {
    console.log(result);
    logger.log(JSON.stringify(result, null, 2));
    logger.success((0, _strings.fillString)('conversion.onConverted', [inFile, outFile.green]));
    return result;
  }).catch(function (error) {
    logger.error(error);
    logger.error(inFile + ' failed to convert');
  });
  logger.print((0, _strings.fillString)('conversion.onConverting', [inFile.cyan]));
}

function validator(inFile, optionalFile, logger) {
  (0, _validate.validate)(inFile, optionalFile, logger).then(function (results) {
    logger.log(JSON.stringify(results[0], null, 2));
    logger.success((0, _strings.fillString)('validation.onValid', [inFile, results[1].green]));
  }).catch(function (error) {
    logger.error((0, _strings.fillString)('validation.onInvalid', [inFile, error]));
  });
  logger.print((0, _strings.fillString)('validation.onValidating', [inFile.cyan]));
}

function validateAction(logger, watching, inFile, optionalFile) {
  if (!inFile) {
    logger.warn('no input file specified');
    return;
  }
  validator(inFile, optionalFile, logger);
  watch(inFile, function (eventType, filename) {
    logger.print((0, _strings.fillString)('validation.onChangeObserved', [inFile]));
    validator(inFile, optionalFile, logger);
  }, watching);
}

function convertAction(logger, watching, inFile, outFile) {
  if (!inFile) {
    logger.warn('no input file specified');
    return;
  }
  converter(inFile, outFile, logger);
  watch(inFile, function (eventType, filename) {
    convert(inFile, outFile, logger);
  }, watching);
}

function startBunsen(commander, processHandle, convertHandler, validateHandler, logger, version) {

  commander.version(version);

  commander.command('convert').alias('c').description('convert old view formats into UI Schema 2').usage('[options] <legacyViewFile> [outputFilePath]').arguments('<legacyViewFile> [viewFile]').option('-v, --verbose', 'more output').option('-w, --watch', 'watch file for changes').action(_lodash2.default.bind(convertHandler, null, logger, commander.watch));

  commander.command('validate').alias('v').description('validate a Bunsen Model or View, or validate a Bunsen View against a Bunsen Model').option('-v, --verbose', 'more output').option('-w, --watch', 'watch file for changes').usage('[options] <viewFile>').usage('[options] <modelFile> [viewFile]').arguments('<modelOrViewFile> [viewFile]').action(_lodash2.default.bind(validateHandler, null, logger, commander.watch));

  commander.parse(processHandle.argv);
  logger.verbose = commander.verbose;
  return commander;
}

var logger = new _logger2.default();

startBunsen(_commander2.default, process, convertAction, validateAction, logger, _package2.default.version);